####Copyleft Ivan Sanchez Fernandez 2017


## ui.R
library(shiny)
library(ggplot2)
library(plotly)

# App appearance
ui <- fluidPage(

  tags$br(),
  tags$strong("COST-EFFECTIVENESS OF GENETIC TESTS FOR EPILEPSY"),
  tags$br(),
  tags$br(),
  tags$strong("Explore how different inputs modify cost-effectiveness"),
  tags$br(),
  tags$header("Remember to enter appropriate inputs into the model: proportions should go from 0 to 1 and costs cannot be negative. With inappropriate inputs the model will return inappropriate outputs or errors."),
  tags$br(),
  
  sidebarLayout(
    sidebarPanel (
  fluidRow(
    
    column(6,
           # Title
           tags$strong("INPUT"),    
           tags$br(),
           tags$br(),
           tags$strong("COST"),
           tags$br(),
           tags$br(),
           tags$br(),
           ## INPUTS
           # Input cost of rescue medications
           # Define the values based on the inputs
           numericInput(inputId = "c.CMA",
                     label = "Chromosomal microarray",
                     value = 1431,
                     min = 0,
                     width = 150),  
           numericInput(inputId = "c.EP",
                     label = "Epilepsy panel",
                     value = 3645,
                     min = 0,
                     width = 150),    
           numericInput(inputId = "c.WES",
                     label = "Whole exome sequencing",
                     value = 6750,
                     min = 0,
                     width = 150)),   

    
    column(6, 
           tags$br(),
           tags$br(),
           tags$strong("GENETIC"),
           tags$br(),
           tags$strong("DIAGNOSIS"),
           tags$br(),
           tags$br(),
           # Input probabilities into the model
           numericInput(inputId = "p.CMA",
                     label = "Chromosomal microarray",
                     value = 0.08,
                     min = 0,
                     max = 1,
                     step = 0.01,
                     width = 150),   
           numericInput(inputId = "p.EP",
                     label = "Epilepsy panel",
                     value = 0.23,
                     min = 0,
                     max = 1,
                     step = 0.01,
                     width = 150),   
           numericInput(inputId = "p.WES",
                     label = "Whole exome sequencing",
                     value = 0.45,
                     min = 0,
                     max = 1,
                     step = 0.01,
                     width = 150))),
  img(src = 'Treeindividual.png', height = "100%", width = "100%")
  ), 
  mainPanel(
    
    
    ## OUTPUTS  
    
    column(12, 
           # Reference to common baseline
           tags$br(),
           tags$strong("OUTPUT"),
           tags$br(),
           tags$header("Cost-effectiveness referenced to common baseline of no genetic testing:"),
           textOutput("ICERbaseline"),
           tags$header("For illustrative purposes only. The most common measure of cost-effectiveness is the ICER referenced to the next most cost-effective alternative as presented in the Cost-Effectiveness Table."),
           
           
           # ICER table
           tags$br(),
           tags$br(),
           tags$strong("COST-EFFECTIVENESS TABLE"),
           tags$br(),
           tags$br(),  
           tableOutput("ICER"),
           tags$header("*Strategies that are not cost-effective dissappear from the table."),
           tags$br(),
           tags$header("Legend: IE: Incremental effectiveness. IC: Incremental cost. ICER: Incremental cost-effectiveness ratio."),
           
           
           #CE graph
           tags$br(),
           tags$br(),
           tags$br(),
           tags$strong("COST-EFFECTIVENESS PLOT"),
           tags$br(),  
           tags$br(), 
           plotlyOutput("plotCE", width = 800, height = 500),
           tags$br(),
           tags$header("Legend: Notest: No genetic testing. CMA: Chromosomal microarray. EP: Epilepsy panel. WES: Whole exome sequencing."))
  )
  ),
  tags$br(),
  tags$br(),
  tags$header("Ivan Sanchez Fernandez 2017. This work is copylefted under a General Public Licence (GPL)."),
  tags$header("Users are fee to run, study, and modify this software and to share and distribute original or modified versions of this software.")
  )






# server.R
library(shiny)
library(ggplot2)
library(plotly)

server <- function(input, output) {
  
  
  ## COST-EFFECTIVENESS REFERENCED TO COMMON BASELINE OF NO GENETIC TESTING
  output$ICERbaseline <- renderText({
    
    # Define the values based on the inputs
    p.CMA <- input$p.CMA
    p.EP <- input$p.EP
    p.WES <- input$p.WES
    
    
    c.CMA <- input$c.CMA
    c.EP <- input$c.EP
    c.WES <- input$c.WES
    
    
    # Define the cost-effectiveness referenced to common baseline of no genetic testing
    CECMA <- c.CMA / p.CMA
    CEEP <- c.EP / p.EP
    CEWES <- c.WES / p.WES

    
    # Save values into a dataframe
    CEbaselinevector <- c(CECMA, CEEP, CEWES)
    CEbaselinenames <- c("CMA", "EP", "WES")
    CEbaseline <- data.frame(CEbaselinenames, CEbaselinevector)
    
    # Order by ascending CE
    CEbaseline <- CEbaseline[order(CEbaseline$CEbaselinevector), ]
    
    # Produce the text
    CEbaseline$CEbaselinenames <- as.character(CEbaseline$CEbaselinenames)
    CEbaseline$CEbaselinevector <- round(CEbaseline$CEbaselinevector, digits = 2)
    CEbaseline$CEbaselinevector <- as.character(CEbaseline$CEbaselinevector)
    finaltext <- capture.output(cat(c(CEbaseline[1, 1], ": ", CEbaseline[1, 2], ";   ",  
                                      CEbaseline[2, 1], ": ", CEbaseline[2, 2], ";   ",
                                      CEbaseline[3, 1], ": ", CEbaseline[3, 2]
                                      ),
                                    sep = ""))
    finaltext
  })
  
  
  
  
  
  
  
  ## INCREMENTAL COST EFFECTIVENESS TABLE
  output$ICER <- renderTable({
    
    # Define the values based on the inputs
    c.Notest <- 0
    c.CMA <- input$c.CMA
    c.EP <- input$c.EP
    c.WES <- input$c.WES

    
    p.Notest <- 0
    p.CMA <- input$p.CMA
    p.EP <- input$p.EP
    p.WES <- input$p.WES

    
    
    
    ## Utility of outcomes
    u.D <- 1
    u.ND <- 0
    
    
    ## Effectiveness into the model
    e.Notest <- u.ND
    e.CMA <- (u.D * p.CMA) + (u.ND * (1 - p.CMA))
    e.EP <- (u.D * p.EP) + (u.ND * (1 - p.EP))
    e.WES <- (u.D * p.WES) + (u.ND * (1 - p.WES))
    
    
    
    ## Create dataframe with cost and effectiveness data
    Cost <- c(c.Notest, c.CMA, c.EP, c.WES)
    Effectiveness <- c(e.Notest, e.CMA, e.EP, e.WES)
    Names <- c("No genetic testing", "Chromosomal microarray", "Epilepsy panel", "Whole exome sequencing")
    genetictests <- data.frame(Names, Cost, Effectiveness)
    
    # Save dataframe for plot
    genetictestsforplot <- genetictests
    
    # Make sure it is numeric
    genetictests$Cost <- as.numeric(genetictests$Cost)
    
    # Sort by cost
    genetictests <- genetictests[order(Cost), ]
    
    
    
  
    
    ### ELIMINATE STRONGLY DOMINATED OPTIONS    
    for (i in 1 : length(genetictests$Cost)) {
      
      # Calculate incremental effectiveness
      for (j in 1 : (length(genetictests$Cost) - 1)){
        genetictests$IE[1] <- genetictests$Effectiveness[1]
        genetictests$IE[j + 1] <- genetictests$Effectiveness[j + 1] - genetictests$Effectiveness[j]
      }
      
      # Identify the first row where the incremental effectiveness is negative
      firstrownegativeIE <- which (genetictests$IE < 0)[1]
      
      # Are there more negatives?
      nomorenegatives <- ifelse(is.na(firstrownegativeIE) == TRUE, 1, 0)
      
      
      
      if (nomorenegatives == 1) {
        break
      } else{
        ## Eliminate the first row where the incremental effectiveness is negative
        genetictests <- genetictests[- firstrownegativeIE, ]
        # Eliminate rows with NA
        genetictests <- genetictests[!is.na(genetictests$Names), ]
        # Renumber the rows
        rownames(genetictests) <- seq(length=nrow(genetictests))
      } 
      
    }
    # Eliminate rows with NA
    genetictests <- genetictests[!is.na(genetictests$Names), ]
    
    
    
    ### CALCULATE ICER AND ELIMINATE OPTIONS BY EXTENDED DOMINANCE
    for (l in 1 : length(genetictests$Cost)){    
      
      # Calculate incremental effectiveness
      for (j in 1 : (length(genetictests$Cost) - 1)){
        genetictests$IE[1] <- genetictests$Effectiveness[1]
        genetictests$IE[j + 1] <- genetictests$Effectiveness[j + 1] - genetictests$Effectiveness[j]
      }
      
      # Calculate incremental cost
      for (k in 1 : (length(genetictests$Cost) - 1)){
        genetictests$IC[1] <- genetictests$Cost[1]
        genetictests$IC[k + 1] <- genetictests$Cost[k + 1] - genetictests$Cost[k]
      }
      
      ## Calculate incremental cost effectiveness
      for (h in 1 : (length(genetictests$Cost) - 1)) {
        genetictests$ICER[1] <- genetictests$IC[1] / genetictests$IE[1]
        genetictests$ICER[h + 1] <- genetictests$IC[h + 1] / genetictests$IE[h + 1]
      }
      
      # Calculate if some ICER is higher than the next
      for (m in 1 : length(genetictests$Cost)) {
        genetictests$ICERdifference[m] <- genetictests$ICER[m + 1] - genetictests$ICER[m]
      }
      
      # Identify the first row where the incremental effectiveness is negative
      firstrownegativeICERdifference <- which (genetictests$ICERdifference < 0)[1]
      
      # Are there more negatives?
      nomorenegativesICERdifference <- ifelse(is.na(firstrownegativeICERdifference) == TRUE, 1, 0)
      
      
      
      if (nomorenegativesICERdifference == 1) {
        break
      } else{
        ## Eliminate the first row where the ICERdifference is negative
        genetictests <- genetictests[- firstrownegativeICERdifference, ]
        # Eliminate rows with NA
        genetictests <- genetictests[!is.na(genetictests$Names), ]
        # Renumber the rows
        rownames(genetictests) <- seq(length=nrow(genetictests))
      }
      
    }
    # Eliminate rows with NA
    genetictests <- genetictests[!is.na(genetictests$Names), ]  
    
    # Eliminate ICERdifference column
    genetictests <- genetictests[ , - 7]
    genetictests
  })

  
  
  
  
  ## PLOT  
  output$plotCE <- renderPlotly({
    
    # Define the values based on the inputs
    c.Notest <- 0
    c.CMA <- input$c.CMA
    c.EP <- input$c.EP
    c.WES <- input$c.WES
    
    
    p.Notest <- 0
    p.CMA <- input$p.CMA
    p.EP <- input$p.EP
    p.WES <- input$p.WES

    
    
    
    ## Utility of outcomes
    u.D <- 1
    u.ND <- 0
    
    
    ## Effectiveness into the model
    e.Notest <- u.ND
    e.CMA <- (u.D * p.CMA) + (u.ND * (1 - p.CMA))
    e.EP <- (u.D * p.EP) + (u.ND * (1 - p.EP))
    e.WES <- (u.D * p.WES) + (u.ND * (1 - p.WES))
    
    
    
    ## Create dataframe with cost and effectiveness data
    Cost <- c(c.Notest, c.CMA, c.EP, c.WES)
    Effectiveness <- c(e.Notest, e.CMA, e.EP, e.WES)
    Names <- c("Notest", "CMA", "EP", "WES")
    genetictests <- data.frame(Names, Cost, Effectiveness)
    
    # Save dataframe for plot
    genetictestsforplot <- genetictests    
    
    
    
    
    # Order factors
    genetictestsforplot$Names <- factor(genetictestsforplot$Names, levels = c("Notest", "CMA", "EP", "WES"))
    # Plot CE
    plotCE <- ggplot(data = genetictestsforplot, aes(x = Effectiveness, y = Cost, 
                                                   color = Names)) +
      geom_point(size = 5, shape = c(18, 15, 17, 16)) +
      theme(panel.background = element_rect(fill = "white"), 
            axis.text = element_text(size = 10, color = "black", face = "bold"),
            axis.title = element_text(size = 16, color = "black", face = "bold"),
            axis.ticks.length = unit(0.2, "cm"), axis.ticks = element_line(size = 1),
            axis.line.x = element_line(color="black", size = 1),
            axis.line.y = element_line(color="black", size = 1), 
            legend.key = element_rect(fill = "white"), legend.position = "bottom") +
      scale_colour_manual(name="",  
                          values = c("Notest"="orange", "CMA"="red", "EP"="blue",
                                     "WES"="darkgreen")) +
      labs(x= "Genetic diagnosis", y= "Cost")
    
    # Make the plot interactive
    plotCE <- ggplotly(plotCE)
    plotCE
  })
  
}



# Run the app
shinyApp(ui = ui, server = server)




